import bpy
from bpy.types import Context, Operator, OperatorProperties

from ..utils.sectionbox import get_sectionbox


class SECTIONBOX_OT_Apply(Operator):
    bl_idname = "sectionbox.apply"
    bl_label = "Apply"
    bl_description = "Apply the section box"
    bl_options = {"UNDO"}

    copy: bpy.props.BoolProperty(
        name="Copy",
        description="Apply the section box as a copy",
        default=False,
    )

    @classmethod
    def description(cls, context: Context, properties: OperatorProperties) -> str:
        if properties.copy:
            return "Apply the section box as a copy"
        else:
            return "Apply the section box"

    def invoke(self, context, event):
        sb = get_sectionbox(context, check_active=False)
        if not sb:
            return {"PASS_THROUGH"}

        name = sb.entry_name
        sb.apply(context, self.copy)
        if self.copy:
            self.report({"INFO"}, message=f'Applied "{name}" as copy successfully!')
        else:
            self.report({"INFO"}, message=f'Applied "{name}" successfully!')
        return {"FINISHED"}
