import bpy
from bpy.types import PropertyGroup


class SECTIONBOX_PG_IncludedObject(PropertyGroup):
    object: bpy.props.PointerProperty(type=bpy.types.Object)
    tmp_object: bpy.props.PointerProperty(type=bpy.types.Object)

    def get_affected_object(self):
        return self.tmp_object if self.tmp_object is not None else self.object

    def remove_tmp_object(self):
        if self.tmp_object:
            bpy.data.meshes.remove(self.tmp_object.data)
            self.tmp_object = None

            try:
                self.object.hide_set(False)
                self.object.hide_render = False
                self.object.hide_viewport = False
            except:
                pass

    def set_tmp_object(self, object, tmp_object_collection):
        if self.tmp_object:
            prev_tmp_object = self.tmp_object
            self.tmp_object = object
            bpy.data.meshes.remove(prev_tmp_object.data)
        elif object:
            self.tmp_object = object
            try:
                self.object.hide_set(True)
                self.object.hide_render = True
                self.object.hide_viewport = True
            except:
                pass
            tmp_object_collection.objects.link(object)
        else:
            self.tmp_object = None

    def delete(self, context, sb):
        object = self.get_affected_object()

        # delete boolean modifier
        mods = [m for m in object.modifiers if m.name.startswith(f"{sb.name}.boolean")]
        for mod in mods:
            object.modifiers.remove(mod)

        # remove section material if one
        section_material = object.data.materials.get(f"{sb.name}.material")
        if section_material:
            mat_id = object.data.materials.find(section_material.name)
            object.data.materials.pop(index=mat_id)

        # remove section node group from materials
        for mat in object.data.materials:
            if not mat:
                continue
            # check if material is still used
            try:
                mat[sb.name] -= 1
            except:
                continue
            if mat[sb.name] != 0:
                continue

            del mat[sb.name]
            node_tree = mat.node_tree
            node_group = node_tree.nodes.get(sb.name)
            if node_group:
                if node_group.inputs[0].links and node_group.outputs[0].links:
                    input_socket = node_group.inputs[0].links[0].from_socket
                    output_socket = node_group.outputs[0].links[0].to_socket
                    node_tree.links.new(input_socket, output_socket)

                node_tree.nodes.remove(node_group)

                for plane in sb.planes:
                    texture_coordinate = node_tree.nodes.get(
                        f"{sb.name}.plane_{plane.id}"
                    )
                    if texture_coordinate:
                        node_tree.nodes.remove(texture_coordinate)

                attribute = node_tree.nodes.get(f"{sb.name}.attribute")
                if attribute:
                    node_tree.nodes.remove(attribute)

            # reset blend method
            if mat.get("prev_blend_method"):
                mat.blend_method = mat["prev_blend_method"]
                del mat["prev_blend_method"]

        # remove section attribute from object
        del object[sb.name]
        self.remove_tmp_object()
